require 'barbeque/exception_handler'
require 'barbeque/message/base'
require 'barbeque/message/invalid_message'
require 'barbeque/message/job_execution'
require 'barbeque/message/job_retry'
require 'barbeque/message/notification'

module Barbeque
  module Message
    class << self
      # @param [Aws::SQS::Types::Message] raw_message
      # @return [Barbeque::Message::Base]
      def parse(raw_message)
        body = JSON.parse(raw_message.body)
        klass = find_class(body['Type'])
        klass.new(raw_message, body)
      rescue JSON::ParserError => e
        ExceptionHandler.handle_exception(e)
        InvalidMessage.new(raw_message, {})
      end

      private

      def find_class(type)
        Message.const_get(type, false)
      rescue NameError => e
        ExceptionHandler.handle_exception(e)
        InvalidMessage
      end
    end
  end
end
